local _G = getfenv(0)
local core
local name, localizedname = GetAddOnInfo("YurrCombatLog")
core = DongleStub("Dongle-1.2"):New(name)
_G.YurrCombatLog = core

local debugf = tekDebug and tekDebug:GetFrame("YurrCombatLog")
local function Debug(...) if debugf then debugf:AddMessage(string.join(", ", ...)) end end

local debugfl = tekDebug and tekDebug:GetFrame("YurrCombatLog-Log")
local function DebugL(...) if debugfl then debugfl:AddMessage(string.join(", ", ...)) end end

local db
function core:Initialize()
    self.db = self:InitializeDB("YurrCombatLogDB", {
		global = {
			outputFrame = "ChatFrame3",
			unitColors = {
				[1297] = "FF75FF", -- self
				[4991] = "804D26", -- pet
				[1394] = "A6FFA6", -- party
				[1396] = "FF7038", -- raid
				[9087] = "404080", -- guardian
				[131072] = "CC54D9", -- focus
				[20334] = "8080FF", -- enemy players/npcs
				[3864] = "FFFFFF", -- outsider
			},
			spellColors = {
				[1] = "FFFFFF", -- physical
				[2] = "FFFF4D", -- holy
				[4] = "FF262E", -- fire
				[8] = "66FF66", -- fire
				[16] = "4D4DE6", -- frost
				[20] = "A63B96", -- frostfire
				[32] = "FFB3FF", -- shadow
				[40] = "B3D9B3", -- natureshadow
				[64] = "BFBFBF", -- arcane
			},
			otherColors = {
				heal = "FFFF4D",
				miss = "FF8080", -- unused for the moment
				buff = "33FF33",
				debuff = "FF3333",
				skill = "FFFF66",
				timestampColor = "737373",
			},
			braceSrcUnit = true,
			leftSrcBrace = "[",
			rightSrcBrace = "]",
			colorSrc = true,
			colorSrcBraces = false,
			leftSrcBraceColor = "FF75FF",
			rightSrcBraceColor = "FF75FF",
			
			prefix = true,
			colorPrefix = true,
			
			braceSpell = true,
			leftSpellBrace = "[",
			rightSpellBrace = "]",
			colorSpell = true,
			colorSpellBraces = false,
			leftSpellBraceColor = "FF75FF",
			rightSpellBraceColor = "FF75FF",
			
			braceDestUnit = true,
			leftDestBrace = "[",
			rightDestBrace = "]",
			colorDest = true,
			colorDestBraces = false,
			leftDestBraceColor = "FF75FF",
			rightDestBraceColor = "FF75FF",
			
			braceAmount = true,
			leftAmountBrace = "[",
			rightAmountBrace = "]",
			colorAmount = true,
			colorAmountBraces = false,
			leftAmountBraceColor = "FF75FF",
			rightAmountBraceColor = "FF75FF",
			overhealing = true,
		},
	}, "global")
    db = core.db[core.db:GetCurrentProfile()]
end

function core:Enable()
	core:RegisterEvent("COMBAT_LOG_EVENT")
end

function core:AffiliationColor(flags)
	for mask in pairs( db.unitColors ) do
		if ( CombatLog_Object_IsA (flags, mask) )then
			color = db.unitColors[mask]
			break
		end
	end
	return color
end

function core:ColorSpellBySchool(school, aura, event)
	if aura == "BUFF" then
		return db.otherColors.buff
	elseif aura =="DEBUFF" then
		return db.otherColors.debuff
	elseif event == "SPELL_HEAL" or event == "SPELL_PERIODIC_HEAL" then
		return db.otherColors.heal
	else
		return db.spellColors[school]
	end
end

function core:COMBAT_LOG_EVENT(...)
	DebugL(1, ...)
	
	local srcUnitName = select(5, ...)
	local srcFlags = select(6, ...)
	local srcUnitNameString
	
	local prefixString
	
	local spellName = select(11, ...)
	local school = select(12, ...)
	local spellNameString
	
	local destUnitName = select(8, ...)
	local destFlags = select(9, ...)
	local destUnitNameString
	
	local amount = select(13, ...)
	local amountString
	
	local finalString
	
	if srcUnitName then
		Debug(1, srcUnitName)
		local str
		local coloredUnitStr
		local leftBrace
		local rightBrace
		if db.colorSrc then
			local color = self:AffiliationColor(srcFlags)
			coloredUnitStr = "|cFF" .. color .. srcUnitName .. "|r"
		end
		if db.colorSrcBraces then
			leftBrace = "|cFF" .. db.leftSrcBraceColor .. db.leftSrcBrace .. "|r"
			rightBrace = "|cFF" .. db.rightSrcBraceColor .. db.rightSrcBrace .. "|r"
		else
			leftBrace = db.leftSrcBrace
			rightBrace = db.rightSrcBrace
		end
		if db.braceSrcUnit then
			str = leftBrace .. "%s" .. rightBrace .. " "
		end
		srcUnitNameString = format(str or "%s ", coloredUnitStr or srcUnitName)
	else
		srcUnitNameString = ""
	end
	
	if db.prefix then
		local str
		local event = select(3, ...)
		local aura = select(13, ...)
		if event == "SPELL_AURA_APPLIED" then
			str = "++"
		elseif event == "SPELL_AURA_REMOVED" then
			str = "--"
		elseif event == "SPELL_AURA_REFRESH" then
			str = "-+"
		elseif event == "SPELL_CAST_START" then
			str = "begins"
		else
			str = ""
		end
		if db.colorPrefix and (str ~= "") then
			local event = select(3, ...)
			local color = self:ColorSpellBySchool(school, aura, event) or "FFFFFF"
			prefixString = format("|cFF%s%s|r ", color, str)
		elseif (str ~= "") then
			prefixString = str .. " "
		else
			prefixString = str
		end
	else
		prefixString = ""
	end
	
	if spellName then
		Debug(1, spellName)
		local str
		local coloredSpellStr
		local leftBrace
		local rightBrace
		if db.colorSpell then
			local event = select(3, ...)
			local aura = select(13, ...)
			local color = self:ColorSpellBySchool(school, aura, event) or "FFFFFF"
			coloredSpellStr = "|cFF" .. color .. spellName .. "|r"
		end
		if db.colorSpellBraces then
			leftBrace = "|cFF" .. db.leftSpellBraceColor .. db.leftSpellBrace .. "|r"
			rightBrace = "|cFF" .. db.rightSpellBraceColor .. db.rightSpellBrace .. "|r"
		end
		if db.braceSpell then
			str = (leftBrace or "") .. "%s" .. (rightBrace or "") .. " "
		end
		spellNameString = format(str or "%s ", coloredSpellStr or spellName)
	else
		spellNameString = ""
	end
	
	if destUnitName then
		Debug(1, destUnitName)
		local str
		local coloredUnitStr
		local leftBrace
		local rightBrace
		if db.colorDest then
			local color = self:AffiliationColor(destFlags)
			coloredUnitStr = "|cFF" .. color .. destUnitName .. "|r"
		end
		if db.colorDestBraces then
			leftBrace = "|cFF" .. db.leftDestBraceColor .. db.leftDestBrace .. "|r"
			rightBrace = "|cFF" .. db.rightDestBraceColor .. db.rightDestBrace .. "|r"
		else
			leftBrace = db.leftDestBrace
			rightBrace = db.rightDestBrace
		end
		if db.braceDestUnit then
			str = leftBrace .. "%s" .. rightBrace .. " "
		end
		destUnitNameString = format(str or "%s ", coloredUnitStr or destUnitName)
	else
		destUnitNameString = ""
	end
	
	if type(amount) == "number" then
		Debug(1, amount)
		local str
		local amountResult
		local coloredAmountStr
		local leftBrace
		local rightBrace
		if db.overhealing then
			local overhealing = select(14, ...)
			amountResult = amount - overhealing .. "(+" .. overhealing .. ")"
		end
		if db.colorAmount then
			local event = select(3, ...)
			local aura = select(13, ...)
			local color = self:ColorSpellBySchool(school, aura, event) or "FFFFFF"
			coloredAmountStr = "|cFF" .. color .. amountResult .. "|r"
		end
		if db.colorAmountBraces then
			leftBrace = "|cFF" .. db.leftAmountBraceColor .. db.leftAmountBrace .. "|r"
			rightBrace = "|cFF" .. db.rightAmountBraceColor .. db.rightAmountBrace .. "|r"
		end
		if db.braceAmount then
			str = (leftBrace or "") .. "%s" .. (rightBrace or "") .. " "
		end
		amountNameString = format(str or "%s ", coloredAmountStr or amountResult)
	else
		amountNameString = ""
	end
	
	finalString = srcUnitNameString .. prefixString .. spellNameString .. destUnitNameString .. amountNameString
	_G[db.outputFrame]:AddMessage(finalString)
end
